@@ -1,19 +1,26 @@
 import os, random, string, json, sys, contextlib, threading
 
 from flask import Flask, send_from_directory, render_template, request
+from contextlib import redirect_stdout
+from io import StringIO
 
 import networkx as nx
 import matplotlib.pyplot as plt
-import io as StringIO
 import tkinter as tk
 
-global global_vars
-global_vars = {}
-global local_vars
-local_vars = {}
+global exec_vars
+exec_vars = {}
 
-global dynamic_cell_output
-dynamic_cell_output = ""
+class DCO:
+
+    def __init__(self, str):
+        self.output = str
+
+    def append(self, str):
+        self.output += str
+
+    def clear(self):
+        self.output = ""
 
 """
 Structure Guide
@@ -86,6 +93,7 @@ class Cell():
     def get_copy(self):
         return Cell(self.name, self.content_type, self.content, self.stdout)
 
+    """
     @contextlib.contextmanager
     def stdoutIO(self, stdout=None):
         old = sys.stdout
@@ -94,9 +102,7 @@ class Cell():
         sys.stdout = stdout
         yield stdout
         sys.stdout = old
-
-    def get_vars(self):
-        return self.self_globals, self.self_locals
+    """
 
     def execute(self):
         # Execute this cell's content
@@ -104,31 +110,26 @@ class Cell():
         if not self.content_type == "python":
             return
 
-        gcopy = global_vars.copy()
-        lcopy = local_vars.copy()
+        global exec_vars
+        ex_vars_copy = exec_vars.copy()
 
         if self.stdout == "internal":
             try:
-                exec(self.content, gcopy, lcopy)
+                exec(self.content, ex_vars_copy)
             except Exception as e:
                 print("Exception occurred in cell " + self.name)
                 print(e)
         elif self.stdout == "external":
             try:
-                with self.stdoutIO() as s:
-                    exec(self.content, gcopy, lcopy)
-                    self.output = s.getvalue()
+                print("<" + self.name + ">")
+                exec(self.content, ex_vars_copy)
             except Exception as e:
                 print("Exception occurred in cell " + self.name)
                 print(e)
         else:
             print("stdout setting \"" + self.stdout + "\" not recognized. Please use internal/external.")
 
-        global_vars.update(gcopy)
-        local_vars.update(lcopy)
-
-        global dynamic_cell_output
-        dynamic_cell_output += self.name + ":\n" + self.output + "\n\n"
+        exec_vars.update(ex_vars_copy)
 
     def __str__(self):
         return self.name + "\n\n" + "```\n" + self.content + "```\n"
@@ -142,13 +143,13 @@ class Graph:
         # Dict to keep track of cell names vs networkx node names
         self.names_to_indeces = {}
         # Dictionaries for variables created by cells
-        global global_vars
-        global_vars = {}
-        global local_vars
-        local_vars = {}
+        global exec_vars
+        exec_vars = {}
         # TextIO object
         self.ti = TextIO()
 
+        self.dco = DCO("")
+
         self.executing = False
 
     def get_lookup_table(self):
@@ -217,6 +218,7 @@ class Graph:
             del self.names_to_indeces[cell_name]
         else:
             print("Cell \"" + cell_name + "\" does not exist.")
+            return -1
 
     def connect_cells(self, idx1, idx2):
         """
@@ -341,8 +343,6 @@ class Graph:
 
             p = threading.Thread(target=cell.execute)
 
-            print(cell.content_type)
-
             if cell.content_type == "python":
                 p.start()
                 p.join()
@@ -353,13 +353,14 @@ class Graph:
             with open(output_filename, 'w') as txt:
                 txt.write(std_file_out)
 
-    def bfs_traversal_execute(self, stdout="internal", output_filename="stdout.txt"):
+    def bfs_traversal_execute(self, stdout="external", output_filename="stdout.txt"):
+        import time
         if len(self.get_all_cells_edges()[0]) == 0:
             return
 
-        std_file_out = ""
-        global dynamic_cell_output
-        dynamic_cell_output = ""
+        self.dco.clear()
+
+        std_file_out = "<root>"
 
         self.executing = True
 
@@ -371,9 +372,14 @@ class Graph:
         root.start()
         root.join()
 
+        time.sleep(.05)
+
+        self.dco.append("<" + root_cell.name + ">\n" + root_cell.output)
+
         std_file_out += root_cell.output
 
-        neighbors = self.graph.neighbors(0)
+        n_ = self.graph.neighbors(0)
+        neighbors = [n for n in n_]
 
         while neighbors:
             new_neighbors = []
@@ -387,20 +393,19 @@ class Graph:
                 neighbor.start()
                 processes.append(neighbor)
 
-                new_neighbors.extend(self.graph.neighbors(n))
+                new_neighbors.extend([i for i in self.graph.neighbors(n)])
 
             [proc.join() for proc in processes]
 
             for n in neighbors:
+                time.sleep(.05)
                 neighbor = self.get_cell(self.get_lookup_table()[n])
+                std_file_out += "<" + neighbor.name + ">\n"
                 std_file_out += neighbor.output
+                self.dco.append("<" + neighbor.name + ">\n" + neighbor.output)
 
             neighbors = new_neighbors
 
-        if stdout == "external":
-            with open(output_filename, 'w') as txt:
-                txt.write(std_file_out)
-
         self.executing = False
 
     def save_graph(self, filename):
@@ -467,10 +472,10 @@ class Interpreter:
         # This will be set if the user executes a .satx file
         self.file = None
         # This determines whether or not stdout gets sent to an external textbox
-        self.stdout = "internal"
+        self.stdout = "external"
         self.stdout_filename = "stdout.txt"
         # Start loop
-        # self.run()
+        self.std_capture = StringIO()
 
     def run_file(self, command):
         """
@@ -684,7 +689,9 @@ class Interpreter:
         """
         to_remove = command[1:]
         for cell in to_remove:
-            self.graph.remove_cell(cell)
+            i = self.graph.remove_cell(cell)
+            if i == -1:
+                return -1
 
     def link(self, command):
         """
@@ -809,10 +816,8 @@ class Interpreter:
 
     def reset_runtime(self):
         # Delete all runtime variables
-        global global_vars
-        global_vars = {}
-        global local_vars
-        local_vars = {}
+        global exec_vars
+        exec_vars = {}
 
     def reset_graph(self, ask=True):
         if(ask):
@@ -820,9 +825,11 @@ class Interpreter:
             if "y" in confirm:
                 self.graph = Graph()
                 self.reset_runtime()
+                self.std_capture = StringIO()
         else:
             self.graph = Graph()
             self.reset_runtime()
+            self.std_capture = StringIO()
 
     def save_graph(self, command):
         """
@@ -837,7 +844,7 @@ interpreter = Interpreter()
 
 def new_name():
     letters_and_digits = string.ascii_letters + string.digits
-    result_str = ''.join((random.choice(letters_and_digits) for i in range(16)))
+    result_str = ''.join((random.choice(letters_and_digits) for i in range(5)))
     return result_str
 
 def create_app(test_config=None):
@@ -874,14 +881,62 @@ def create_app(test_config=None):
     def index_style():
         return render_template("index_style.css")
 
+    @app.route("/rubyblue.css")
+    def rubyblue():
+        return render_template("rubyblue.css")
+
     @app.route("/canvas_style.css")
     def canvas_style():
         return render_template("canvas_style.css")
 
+    @app.route("/bootstrap.min.css")
+    def bootstrap_style():
+        return render_template("bootstrap.min.css")
+
+    @app.route("/gh-buttons.css")
+    def gh_buttons():
+        return render_template("gh-buttons.css")
+
+    @app.route("/static/css/bootstrap.min.css.map")
+    def bootstrap_map():
+        return "idk why this is here but this hides an error"
+
     @app.route("/script.js")
     def script_js():
         return render_template("script.js")
 
+    @app.route("/jquery-1.12.4.js")
+    def jquery_js():
+        return render_template("jquery-1.12.4.js")
+
+    @app.route("/jquery-ui.js")
+    def jquery_ui_js():
+        return render_template("jquery-ui.js")
+
+    @app.route("/jquery-ui.css")
+    def jquery_ui_css():
+        return render_template("jquery-ui.css")
+
+    @app.route("/panzoom.min.js")
+    def panzoom_js():
+        return render_template("panzoom.min.js")
+
+    @app.route("/codemirror.css")
+    def codemirror_css():
+        return render_template("codemirror.css")
+
+    @app.route("/codemirror/lib/codemirror.js")
+    def codemirror_js():
+        return render_template("codemirror/lib/codemirror.js")
+
+    @app.route("/python.js")
+    def py_js():
+        return render_template("python.js")
+
+    @app.route("/markdown.js")
+    def md_js():
+        return render_template("markdown.js")
+
     @app.route("/create_cell/", methods=["GET"])
     def create_cell():
         name = new_name()
@@ -892,15 +947,20 @@ def create_app(test_config=None):
     def destroy_cell():
         cell_name = request.get_json().strip()
 
+        cell = interpreter.graph.get_cell(cell_name)
+
         initial_length = len(interpreter.graph.graph.nodes())
 
-        interpreter.remove_cell(["remove", cell_name])
+        output = interpreter.remove_cell(["remove", cell_name])
 
         success = "false"
-        if initial_length == len(interpreter.graph.graph.nodes()):
+        if not initial_length == len(interpreter.graph.graph.nodes()):
             success = "true"
 
-        return success
+        return {'success': success,
+                'name': cell.name,
+                'content': cell.content,
+                'content_type': cell.content_type}
 
     @app.route("/edit_cell/", methods=["POST"])
     def edit_cell():
@@ -966,8 +1026,9 @@ def create_app(test_config=None):
 
     @app.route("/bfs_execute/", methods=["POST"])
     def bfs_execute():
-        interpreter.execute(["execute"])
-
+        interpreter.std_capture = StringIO()
+        with redirect_stdout(interpreter.std_capture):
+            interpreter.execute(["execute"])
         return "true"
 
     @app.route("/shutdown/", methods=["POST"])
@@ -1013,11 +1074,9 @@ def create_app(test_config=None):
 
     @app.route("/dynamic_cell_output/", methods=["GET"])
     def get_dynamic_cell_output():
-        global dynamic_cell_output
         if interpreter.graph.executing:
-            return render_template("dynamic_cell_output.html", dynamic_cell_output=dynamic_cell_output)
-        print(dynamic_cell_output)
-        return "done_executing;" + dynamic_cell_output
+            return interpreter.std_capture.getvalue()
+        return "<!--SATYRN_DONE_EXECUTING-->" + interpreter.std_capture.getvalue() + ("<execution complete>" if len(interpreter.std_capture.getvalue()) > 0 else "")
 
     @app.route("/load_graph/", methods=["POST"])
     def load_graph():
@@ -1067,5 +1126,28 @@ def create_app(test_config=None):
         interpreter.create_cell(["create_cell", "root", "python", "n"])
         return "true"
 
+    @app.route("/child_cell/", methods=["POST"])
+    def add_child():
+        parent_name = request.get_json()['parent_name'].strip()
+        child_name = new_name()
+
+        interpreter.create_cell(["create_cell", child_name, "python", "n"])
+        interpreter.link(['link', parent_name, child_name])
+
+        return child_name
+
+    @app.route("/individual_execute/", methods=["POST"])
+    def individual_execute():
+        cell_name = request.get_json()['cell_name'].strip()
+        with redirect_stdout(interpreter.std_capture):
+            interpreter.execute(["execute", cell_name])
+
+        return "true"
+
+    @app.route("/clear_output/", methods=["POST"])
+    def clear_dco():
+        interpreter.std_capture = StringIO()
+
+        return "true"
 
     return app
\ No newline at end of file
