@@ -1,5 +1,14 @@
 var filename = "Untitled.SATX";
 var num_cells = 1;
+var is_executing = false;
+var just_finished = false;
+
+setup_keyboard_shortcuts(document);
+
+var last_deleted_cell = {'name': '',
+                         'content': '',
+                         'content_type': ''
+};
 
 $(window).load(function () {
     $.ajax({
@@ -22,13 +31,23 @@ $(window).load(function () {
                 if(names.length == 0){
                     return;
                 }
-                for(var i = 0; i < num_cells; i++){
-                    $("iframe").contents().find("#draggable").remove();
-                }
-                num_cells = 0;
+                removeDraggables();
                 for(var i = 0; i < names.length; i++){
                     create_cell($("iframe").contents(), names[i], contents[i], content_types[i]);
                 }
+
+                $.ajax({
+                    type : "GET",
+                    url : "/dynamic_cell_output/",
+                    success: function (data) {
+                        if(data.includes("<!--SATYRN_DONE_EXECUTING-->")){
+                            is_executing = false;
+                            just_finished = true;
+                            data = data.substring(28);
+                        }
+                        updateDCO(data);
+                    }
+                });
             }
         }
     });
@@ -63,16 +82,26 @@ $("#file-input").change(function(e){
                     alert("Loading error: names and contents are not congruent")
                 }
                 else{
-                    for(var i = 0; i < num_cells; i++){
-                        $("iframe").contents().find("#draggable").remove();
-                    }
-                    num_cells = 0;
-
+                    removeDraggables();
                     for(var i = 0; i < names.length; i++){
                         create_cell($("iframe").contents(), names[i], contents[i], content_types[i]);
                     }
 
                     $("#file-input").val(null);
+                    /*
+                    $.ajax({
+                        type : "GET",
+                        url : "/dynamic_cell_output/",
+                        success: function (data) {
+                            if(data.includes("<!--SATYRN_DONE_EXECUTING-->")){
+                                is_executing = false;
+                                just_finished = true;
+                                data = data.substring(28);
+                            }
+                            updateDCO(data);
+                        }
+                    });
+                    */
                 }
             }
         });
@@ -99,6 +128,27 @@ $("#graph_name_p").on("click", function(){
 $("iframe").load(function(){
     var doc = $(this).contents();
 
+    setup_keyboard_shortcuts(doc);
+
+    $(doc).delegate('textarea', 'keydown', function(e) {
+        var keyCode = e.keyCode || e.which;
+
+        if (keyCode == 9) {
+            e.preventDefault();
+            var start = this.selectionStart;
+            var end = this.selectionEnd;
+
+            // set textarea value to: text before caret + tab + text after caret
+            $(this).val($(this).val().substring(0, start)
+                + "\t"
+                + $(this).val().substring(end));
+
+            // put caret at right position again
+            this.selectionStart =
+                this.selectionEnd = start + 1;
+        }
+    });
+
     //panzoom scene
     //var element = doc.querySelector('#scene');
 
@@ -169,11 +219,17 @@ $("iframe").load(function(){
                     dataType: "json",
                     data: JSON.stringify(right_clicked_cell),
                     contentType: "application/json",
-                    success: function (success) {
+                    complete: function (o) {
+                        var output = o.responseJSON;
+
+                        var success = output['success']
                         if(success == "false"){
                             alert("Couldn't remove cell " + right_clicked_cell);
                         }
                         else{
+                            last_deleted_cell['name'] = output['name'];
+                            last_deleted_cell['content'] = output['content'];
+                            last_deleted_cell['content_type'] = output['content_type'];
                             doc.find("div").remove("." + right_clicked_cell);
                         }
                     }
@@ -184,7 +240,7 @@ $("iframe").load(function(){
                     type : "POST",
                     url : '/root_has_outputs/',
                     complete: function (s) {
-                        if(s['responseText'] == "warning"){
+                        if(s['responseText'] == "warning" && num_cells > 1){
                             if(confirm("The root node has no outward links. Code execution starts at root, so this may result in unintended consequences. Continue?")){
                                 bfs_execute();
                             }
@@ -212,6 +268,32 @@ $("iframe").load(function(){
                         create_cell(doc, data['cell_name'], data['content'], data['content_type']);
                     }
                 });
+                break;
+            case "child_cell":
+                $.ajax({
+                    type : "POST",
+                    url : '/child_cell/',
+                    dataType: "json",
+                    data: JSON.stringify({'parent_name': right_clicked_cell}),
+                    contentType: "application/json",
+                    complete: function(data){
+                        create_cell(doc, data['responseText'], "", "python");
+                    }
+                });
+                break;
+            case "individual_execute":
+                $.ajax({
+                    type : "POST",
+                    url : '/individual_execute/',
+                    dataType: "json",
+                    data: JSON.stringify({'cell_name': right_clicked_cell}),
+                    contentType: "application/json",
+                    complete: function(){
+                        is_executing = true;
+                        just_finished = false;
+                    }
+                });
+                break;
         }
 
         // Hide it AFTER the action was triggered
@@ -226,13 +308,16 @@ $("iframe").load(function(){
 
             // Hide it
             doc.find(".custom-menu").hide(100);
+
+            right_clicked_cell = "";
+            clicked_textarea = "";
         }
     });
 
     //when a textarea is clicked
     doc.on("click", "textarea", (function(){
         started_ta_edit = true;
-        clicked_textarea = $(this).attr("class").substring(9);
+        clicked_textarea = $(this).attr("id").substring(9);
     }));
 
     //linking logic
@@ -296,6 +381,9 @@ $("iframe").load(function(){
     doc.on("click", "h6", function(){
         var old_name = $(this).text();
         var new_name = prompt("Rename cell: ");
+        if(new_name == null){
+            return;
+        }
         if(new_name.length < 1){
             alert("Please use a longer cell name");
             return;
@@ -329,7 +417,7 @@ $("iframe").load(function(){
                             else{
                                 h6.html(new_name);
                                 doc.find("." + old_name).attr('class', new_name + " " + doc.find("." + old_name).attr('class').replace(old_name, ""));
-                                doc.find(".textarea_" + old_name).addClass("textarea_" + new_name).removeClass("textarea_" + old_name);
+                                doc.find("#textarea_" + old_name).attr("id", "textarea_" + new_name);
                             }
                         }
                     });
@@ -338,25 +426,6 @@ $("iframe").load(function(){
             }
         });
     })
-
-    doc.on("input propertychange", 'textarea', function() {
-        currentVal = $(this).val();
-        $.ajax({
-            type : "POST",
-            url : '/edit_cell/',
-            dataType: "json",
-            data: JSON.stringify({'name': clicked_textarea,
-                'content': currentVal}),
-            contentType: "application/json",
-            success: function (success) {
-                if(success == "false"){
-                    alert("Couldn't edit cell " + right_clicked_cell);
-                }
-            }
-        });
-
-        started_ta_edit = false;
-    });
 });
 
 $(document).on("click", ".shutdown", function(){
@@ -364,8 +433,15 @@ $(document).on("click", ".shutdown", function(){
 })
 
 //Navbar stuff
-$(document).on("click", "a", function(){
-    var attr = $(this).attr("action");
+$(document).on("click", "a, li", function(){
+    var attr = "";
+    var child = $(this).children()[0];
+    if(child === 'undefined'){
+        attr = $(this).attr("action");
+    }
+    else{
+        attr = $(child).attr("action");
+    }
 
     switch(attr){
         case "shutdown":
@@ -376,7 +452,7 @@ $(document).on("click", "a", function(){
                 type : "POST",
                 url : '/root_has_outputs/',
                 complete: function (s) {
-                    if(s['responseText'] == "warning"){
+                    if(s['responseText'] == "warning" && num_cells > 1){
                         if(confirm("The root node has no outward links. Code execution starts at root, so this may result in unintended consequences. Continue?")){
                             bfs_execute();
                         }
@@ -397,7 +473,6 @@ $(document).on("click", "a", function(){
                 contentType: "application/json",
                 complete: function (s) {
                     satx_text = s['responseText'];
-                    console.log(satx_text);
 
                     var dwnld_ele = document.createElement('a');
                     dwnld_ele.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURI(satx_text));
@@ -449,8 +524,20 @@ $(document).on("click", "a", function(){
                     data: JSON.stringify({'cell_name': clicked_textarea}),
                     contentType: "application/json",
                     success: function (data) {
-                        var ele = $("iframe").contents().find(".textarea_" + clicked_textarea).prev();
+                        var ele = $("iframe").contents().find("#textarea_" + clicked_textarea);
                         ele.removeClass().addClass("highlightGreen");
+
+                        for(var i = 0; i < codemirrors.length; i++){
+                            var dict = codemirrors[i];
+                            var n = dict['name'];
+                            if(n != clicked_textarea){
+                                continue;
+                            }
+                            var cm = dict['codemirror'];
+                            cm.setOption("mode", "markdown");
+                        }
+
+                        console.log(codemirrors);
                     }
                 });
             }
@@ -464,8 +551,18 @@ $(document).on("click", "a", function(){
                     data: JSON.stringify({'cell_name': clicked_textarea}),
                     contentType: "application/json",
                     success: function (data) {
-                        var ele = $("iframe").contents().find(".textarea_" + clicked_textarea).prev();
+                        var ele = $("iframe").contents().find("#textarea_" + clicked_textarea);
                         ele.removeClass().addClass("highlightBlue");
+
+                        for(var i = 0; i < codemirrors.length; i++){
+                            var dict = codemirrors[i];
+                            var n = dict['name'];
+                            if(n != clicked_textarea){
+                                continue;
+                            }
+                            var cm = dict['codemirror'];
+                            cm.setOption("mode", "python");
+                        }
                     }
                 });
             }
@@ -492,7 +589,7 @@ $(document).on("click", "a", function(){
                             type : "POST",
                             url : '/root_has_outputs/',
                             complete: function (s) {
-                                if(s['responseText'] == "warning"){
+                                if(s['responseText'] == "warning" && num_cells > 1){
                                     if(confirm("The root node has no outward links. Code execution starts at root, so this may result in unintended consequences. Continue?")){
                                         bfs_execute();
                                     }
@@ -506,6 +603,38 @@ $(document).on("click", "a", function(){
                 });
             }
             break;
+        case "clear_dco":
+            $.ajax({
+                type : "POST",
+                url : '/clear_output/',
+                complete: function () {
+                    updateDCO("");
+                }
+            });
+            break;
+        case "run_cell":
+            $.ajax({
+                type : "POST",
+                url : '/individual_execute/',
+                dataType: "json",
+                data: JSON.stringify({'cell_name': clicked_textarea}),
+                contentType: "application/json",
+                complete: function(){
+                    is_executing = true;
+                    just_finished = false;
+                }
+            });
+            break;
+        case "create_cell":
+            $.ajax({
+                type : "GET",
+                url : '/create_cell/',
+                dataType: "text",
+                success: function (data) {
+                    create_cell($("iframe").contents(), data, "", "python");
+                }
+            });
+            break;
     }
 })
 
@@ -529,6 +658,8 @@ var started_ta_edit = false;
 var attempting_to_link = false;
 
 function bfs_execute(){
+    is_executing = true;
+    just_finished = false;
     $.ajax({
         type : "POST",
         url : '/bfs_execute/',
@@ -540,36 +671,46 @@ function bfs_execute(){
     });
 }
 
-function create_cell(doc, name, content, contentType){
+var codemirrors = [];
 
+function create_cell(doc, name, content="\n\n\n", contentType){
     if(num_cells == 0){
         pageX = 0;
         pageY = 0;
     }
 
-    var colorClass = "";
-    switch(contentType){
-        case "python":
-            colorClass = "Blue";
-            break;
-        case "markdown":
-            colorClass = "Green";
-            break;
-        case "root":
-            colorClass = "Root";
-            break;
-    }
+    var colorClass = contentType == "python" ? "Blue" : "Green";
 
-    //    var new_ele = '<div id="draggable" class="' + name + '"><h6 class="label">' + name + '</h6><div class="draggable"><div class="' + colorClass + '"</div><textarea class="textarea_' + name + '" spellcheck="false">' + content + '</textarea></div></div>'
+    doc.find("#scene").append('<div id="draggable" class="'.concat(name, '">' +
+        '<div style="display: inline-block; margin-right: 10px;">' +
+            '<h6 class="label" id="running_' + name + '" style="width: auto">[ ]</h6>' +
+        '</div>' +
+        '<div style="display: inline-block;">' +
+            '<h6 class="label">' + name + '</h6>' +
+        '</div>' +
+        '<div class="draggable" style="border-radius: 5px;">' +
+            '<div id="textarea_' + name + '" class="highlight' + colorClass + '" style="border-radius: 5px;"></div>' +
+        '</div></div>'))
 
-    doc.find("#scene").append('<div id="draggable" class="'.concat(name, '"><h6 class="label">' + name + '</h6><div class="draggable" style="border-radius: 5px;"><div class="highlight' + colorClass + '" style="border-radius: 5px;"></div><textarea class="textarea_' + name + '" spellcheck="false">' + content + '</textarea></div></div>'))
-
-    doc.find(".".concat(name)).css("top", (Math.ceil(pageY / 30 )*30)-4 );
-    doc.find(".".concat(name)).css("left", (Math.ceil(pageX / 30 )*30)-4 );
+    doc.find(".".concat(name)).css("top", (Math.ceil(pageY / 30 )*30)+10 );
+    doc.find(".".concat(name)).css("left", (Math.ceil(pageX / 30 )*30)+10 );
 
     //Grid system
     doc.find(".".concat(name)).draggable({ snap: ".".concat(name), grid: [ 30, 30 ] });
 
+    //Codemirror
+    var cm = add_codemirror_editor(document.getElementById("canvas").contentWindow.document,"#textarea_" +  name, content, contentType);
+    $('.CodeMirror').resizable({
+        resize: function() {
+            cm.setSize($(this).width(), $(this).height());
+        }
+    });
+    doc.find(".CodeMirror").css("margin-top", "11px");
+
+    var dict = {'name': name, codemirror: cm};
+
+    codemirrors.push(dict);
+
     num_cells += 1;
 }
 
@@ -584,4 +725,247 @@ function shutdown(){
             }
         });
     }
+}
+
+function removeDraggables(){
+    for(var i = 0; i < num_cells; i++){
+        $("iframe").contents().find("#draggable").remove();
+    }
+    num_cells = 0;
+}
+
+function updateDCO(appended_string){
+    $("#dynamic_code_output").val(appended_string);
+}
+
+const throttle = (callback, delay) => {
+    let throttleTimeout = null;
+    let storedEvent = null;
+
+    const throttledEventHandler = event => {
+        storedEvent = event;
+
+        const shouldHandleEvent = !throttleTimeout;
+
+        if (shouldHandleEvent) {
+            callback(storedEvent);
+
+            storedEvent = null;
+
+            throttleTimeout = setTimeout(() => {
+                throttleTimeout = null;
+
+                if (storedEvent) {
+                    throttledEventHandler(storedEvent);
+                }
+            }, delay);
+        }
+    };
+
+    return throttledEventHandler;
+};
+
+const debounce = (func, wait) => {
+    let timeout;
+
+    return function executedFunction(...args) {
+        const later = () => {
+            clearTimeout(timeout);
+            func(...args);
+        };
+
+        clearTimeout(timeout);
+        timeout = setTimeout(later, wait);
+    };
+};
+
+var throttle_func = throttle(function() {
+    if(is_executing){
+        $.ajax({
+            type : "GET",
+            url : "/dynamic_cell_output/",
+            success: function (data) {
+                if(data.includes("<!--SATYRN_DONE_EXECUTING-->")){
+                    is_executing = false;
+                    just_finished = true;
+                    data = data.substring(28);
+                }
+                updateDCO(data);
+            }
+        });
+    }
+    else if(just_finished){
+        $.ajax({
+            type : "GET",
+            url : "/dynamic_cell_output/",
+            success: function (data) {
+                if(data.includes("<!--SATYRN_DONE_EXECUTING-->")){
+                    data = data.substring(28);
+                }
+                updateDCO(data);
+            }
+        });
+        just_finished = false;
+        is_executing = false;
+    }
+}, 2000);
+
+var debounce_func = debounce(function(){
+    if(is_executing){
+        $.ajax({
+            type : "GET",
+            url : "/dynamic_cell_output/",
+            success: function (data) {
+                if(data.includes("<!--SATYRN_DONE_EXECUTING-->")){
+                    is_executing = false;
+                    just_finished = true;
+                    data = data.substring(28);
+                }
+                updateDCO(data);
+            }
+        });
+    }
+    else if(just_finished){
+        $.ajax({
+            type : "GET",
+            url : "/dynamic_cell_output/",
+            success: function (data) {
+                if(data.includes("<!--SATYRN_DONE_EXECUTING-->")){
+                    data = data.substring(28);
+                }
+                updateDCO(data);
+            }
+        });
+        just_finished = false;
+        is_executing = false;
+    }
+}, 500);
+
+window.setInterval(function(){
+    debounce_func();
+}, 1000);
+
+function add_codemirror_editor(doc, ta_id, value='\n\n', contentType){
+    var cm = CodeMirror(doc.querySelector(ta_id), {
+        lineNumbers: true,
+        tabSize: 4,
+        indentUnit: 4,
+        value: value,
+        mode: contentType,
+        theme: 'rubyblue'
+    });
+    cm.on("change", function(){
+        currentVal = cm.getValue();
+        $.ajax({
+            type : "POST",
+            url : '/edit_cell/',
+            dataType: "json",
+            data: JSON.stringify({'name': clicked_textarea,
+                'content': currentVal}),
+            contentType: "application/json",
+            success: function (success) {
+                if(success == "false"){
+                    alert("Couldn't edit cell " + right_clicked_cell);
+                }
+            },
+            statusCode: {
+                500: function() {
+                    if(confirm("It seems there was a disconnect. Do you want to reconnect?")){
+                        location.reload();
+                    }
+                }
+            }
+        });
+
+        started_ta_edit = false;
+    });
+
+    return cm;
+}
+
+function setup_keyboard_shortcuts(doc){
+    $(doc).bind('keyup', 'Ctrl+return', function(){
+        if(clicked_textarea == ""){
+            return;
+        }
+        $.ajax({
+            type : "POST",
+            url : '/individual_execute/',
+            dataType: "json",
+            data: JSON.stringify({'cell_name': clicked_textarea}),
+            contentType: "application/json",
+            complete: function(){
+                is_executing = true;
+                just_finished = false;
+            }
+        });
+    });
+    $(doc).bind('keyup', 'Ctrl+r', function(){
+        bfs_execute();
+    });
+    $(doc).bind('keyup', 'Ctrl+s', function(){
+        var satx_text = "";
+        $.ajax({
+            type : "POST",
+            url : '/get_satx_text/',
+            dataType: "json",
+            data: JSON.stringify({'text': ""}),
+            contentType: "application/json",
+            complete: function (s) {
+                satx_text = s['responseText'];
+
+                var dwnld_ele = document.createElement('a');
+                dwnld_ele.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURI(satx_text));
+                dwnld_ele.setAttribute('download', filename);
+
+                dwnld_ele.style.display = 'none';
+                document.body.appendChild(dwnld_ele);
+                dwnld_ele.click();
+                document.body.removeChild(dwnld_ele);
+            }
+        });
+    });
+    $(doc).bind('keyup', 'Ctrl+c', function(){
+        console.log("Interrupt");
+    });
+    $(doc).bind('keyup', 'Ctrl+d', function(){
+        $.ajax({
+            type : "POST",
+            url : '/dupe_cell/',
+            dataType: "json",
+            data: JSON.stringify({'cell_name': clicked_textarea,
+                'content': '',
+                'content_type': ''}),
+            contentType: "application/json",
+            success: function (data) {
+                create_cell(doc, data['cell_name'], data['content'], data['content_type']);
+            }
+        });
+    });
+    $(doc).bind('keyup', 'Ctrl+backspace', function(){
+        if(clicked_textarea == ""){
+            return;
+        }
+        $.ajax({
+            type : "POST",
+            url : '/destroy_cell/',
+            dataType: "json",
+            data: JSON.stringify(clicked_textarea),
+            contentType: "application/json",
+            complete: function (o) {
+                var output = o.responseJSON;
+                var success = output['success']
+                console.log(output);
+                if(success == "false"){
+                    alert("Couldn't remove cell " + clicked_textarea);
+                }
+                else{
+                    last_deleted_cell['name'] = output['name'];
+                    last_deleted_cell['content'] = output['content'];
+                    last_deleted_cell['content_type'] = output['content_type'];
+                    doc.find("div").remove("." + clicked_textarea);
+                }
+            }
+        });
+    });
 }
\ No newline at end of file
